#include "stdio.h"
#include "stdlib.h"
#include "../../src/aria.hpp"

void printBlockOfLength(unsigned char *b, int len) {
	int i;
  
	for (i=0; i<len; i++, b++) {
		printf("%02x", *b);
		if (i%4==3 && i<len-1) printf(" ");
	}
}

void printBlock(unsigned char *b) {
  printBlockOfLength(b, 16);
}

#define ENC_TEST_KEY_BITS 192
void enc_test()
{
  unsigned char mk[ENC_TEST_KEY_BITS / 8];
  unsigned char p[ARIA_NB] = {
    0x11, 0x11, 0x11, 0x11, 0xaa, 0xaa, 0xaa, 0xaa,
    0x11, 0x11, 0x11, 0x11, 0xbb, 0xbb, 0xbb, 0xbb};
  unsigned char c[ARIA_NB];
  unsigned char cryptResult[ARIA_NB] = {
    0x8d, 0x14, 0x70, 0x62, 0x5f, 0x59, 0xeb, 0xac,
    0xb0, 0xe5, 0x5b, 0x53, 0x4b, 0x3e, 0x46, 0x2b};  
  int i, flag;

	for (i=0; i<16; i++) mk[i]=i*0x11;
	for (i=16; i<24; i++) mk[i]=(i-16)*0x11;

  Aria aria;
  aria.initialize(mk, ENC_TEST_KEY_BITS, true);
  aria.crypt(p, c, ARIA_NB);

  printf("BEGIN testing basic encryption...\n");
  printf("Testing whether the encryption would come out correctly, \
for 14-round ARIA.\n");
  printf("key      : "); printBlockOfLength(mk, 24); printf("\n");
  printf("plaintext: "); printBlock(p); printf("\n");
  printf("result is: "); printBlock(c); printf("\n");
  printf("should be: "); printBlock(cryptResult); printf("\n");
  flag=0;
  for (i=0; i<16; i++)
    if (c[i]!=cryptResult[i])
      flag=1;
  if (flag==1)
	  printf("The result is incorrect!\n");
  else
	  printf("Okay.  The result is correct.\n");
  printf("END   testing basic encryption.\n\n");
}

#define DEC_TEST_KEY_BITS 256
void dec_test()
{
  unsigned char p[ARIA_NB];
  unsigned char c[ARIA_NB];
  int i, flag;

  unsigned char mk[DEC_TEST_KEY_BITS / 8];
  for (i=0; i<32; i++) mk[i]=0;
  for (i=0; i<16; i++) p[i]=0;
  Aria aria;
  
  printf("BEGIN testing the roundtrip...\n");
  printf("For key size of 256 bits, starting with \
the zero plaintext and the zero key, let's see if \
we may recover the plaintext by decrypting the \
encrypted ciphertext.\n");
  aria.initialize(mk, DEC_TEST_KEY_BITS, true);
  printf("plaintext : "); printBlock(p); printf("\n");
  aria.crypt(p, c, ARIA_NB);
  printf("ciphertext: "); printBlock(c); printf("\n");
  aria.initialize(mk, DEC_TEST_KEY_BITS, false);
  aria.crypt(c, p, ARIA_NB);
  printf("decrypted : "); printBlock(p); printf("\n");
  flag=0;
  for (i=0; i<16; i++)
    if (p[i]!=0)
      flag=1;
  if (flag==1)
	  printf("The result is incorrect!\n");
  else
	  printf("Okay.  The result is correct.\n");
  printf("END   testing the roundtrip.\n");
}

int main(int argc, char **argv)
{
  enc_test();
  dec_test();
  return 0;
}