//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
#include "FormAbout.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
void TMainForm::loadControl()
{
  if (!global->loadWindowPos(Handle, "MainForm"))
  {
    Left = 0;
    Top = 0;
  }

  VDXML* xml = global->xmlFile.root.gotoChild("Aria");
  cbxKeyBits->ItemIndex = xml->getInt("KeyBits", -1);
  cbxCryptType->ItemIndex = xml->getInt("CryptType", -1);
  edMasterKey->Text = xml->getStr("MasterKey", "").c_str();
  mmInput->Lines->Text = xml->getStr("Input", "").c_str();
  mmOutput->Lines->Text = xml->getStr("Output", "").c_str();
}

void TMainForm::saveControl()
{
  global->saveWindowPos(Handle, "MainForm");

  VDXML* xml = global->xmlFile.root.gotoChild("Aria");
  xml->setInt("KeyBits", cbxKeyBits->ItemIndex);
  xml->setInt("CryptType", cbxCryptType->ItemIndex);
  xml->setStr("MasterKey", edMasterKey->Text.c_str());
  AnsiString s = mmInput->Lines->Text;
  xml->setStr("Input", s.c_str());
  s = mmOutput->Lines->Text;
  xml->setStr("Output", s.c_str());
}

void TMainForm::setControl()
{
  //
  // Initialize
  //
  if (initialized)
  {
    btnInitialize->Enabled = false;
    cbxKeyBits->Enabled = false;
    cbxCryptType->Enabled = false;
    edMasterKey->Enabled = false;
    btnGenerateRandomMasterKey->Enabled = false;
  } else
  {
    btnInitialize->Enabled =
      (cbxKeyBits->ItemIndex !=-1) &&
      (cbxCryptType->ItemIndex != -1) &&
      (edMasterKey->Text.Length() == Aria::getMasterKeySize(StrToIntDef(cbxKeyBits->Text, 0)) * 2);
    cbxKeyBits->Enabled = true;
    cbxCryptType->Enabled = true;
    edMasterKey->Enabled = true;
    btnGenerateRandomMasterKey->Enabled = true;
  }

  //
  // Finalize
  //
  btnFinalize->Enabled = initialized;

  //
  // Crypt
  //
  btnCrypt->Enabled = initialized &&
    (mmInput->Lines->Text.Length() % (ARIA_NB * 2) == 0);
  mmInput->Enabled = initialized;
  mmOutput->Enabled = initialized;
}

int TMainForm::bin_to_hex(unsigned char* hex, unsigned char* bin, int binLen)
{
  int res = 0;

  for (int i = 0; i < binLen; i++)
  {
    unsigned char temp[3];
    sprintf(temp, "%02X", *bin++);
    *hex++ = temp[0];
    *hex++ = temp[1];
    res++;
  }
  return res;
}

int TMainForm::hex_to_bin(unsigned char* bin, unsigned char* hex, int hexLen)
{
  int res = 0;

  for (int i = 0; i < hexLen / 2; i++)
  {
    AnsiString s = "0X";
    s += (char)hex[0];
    s += (char)hex[1];
    int j = StrToInt(s);
    *bin++ = (unsigned char)j;
    hex += 2;
    res++;
  }
  return res;
}

//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
  : TForm(Owner)
{
  initialized = false;
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::FormCreate(TObject *Sender)
{
//  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormShow(TObject *Sender)
{
  loadControl();
  setControl();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
  saveControl();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::btnInitializeClick(TObject *Sender)
{
  int keyBits = StrToInt(cbxKeyBits->Text);
  int keyBytes = keyBits / 8;
  unsigned char* mk = new unsigned char[keyBytes];
  hex_to_bin(mk, edMasterKey->Text.c_str(), edMasterKey->Text.Length());

  bool encrypt = cbxCryptType->ItemIndex == 0;
  if (aria.initialize(mk, keyBits, encrypt))
    initialized = true;
  delete[] mk;
  setControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::btnGenerateRandomMasterKeyClick(TObject *Sender)
{
  int keyBits = StrToInt(cbxKeyBits->Text);
  int keyBytes = keyBits / 8;
  unsigned char* mk = new unsigned char[keyBytes];

  for (int i = 0; i < keyBytes; i++) mk[i] = random(256);
  unsigned char* hex = new unsigned char[keyBytes * 2 + 1];
  bin_to_hex(hex, mk, keyBytes);
  hex[keyBytes * 2] = '\0';

  edMasterKey->Text = (char*)hex;

  delete[] mk;
  delete[] hex;

  setControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::btnFinalizeClick(TObject *Sender)
{
  if (aria.finalize())
    initialized = false;
  setControl();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::btnCryptClick(TObject *Sender)
{
  int inputSize = mmInput->Lines->Text.Length() / 2;

  if (inputSize % ARIA_NB != 0)
  {
    ShowMessage(format("Length must be multiple of %d", ARIA_NB).c_str());
    return;
  }

  unsigned char* input = new unsigned char[inputSize];
  hex_to_bin(input, mmInput->Lines->Text.c_str(), inputSize * 2);

  unsigned char* output = new unsigned char[inputSize];
  int res = aria.crypt(input, output, inputSize);
  if (res == -1)
  {
    ShowMessage("Can not crypt");
  }

  unsigned char* hex = new unsigned char[res * 2 + 1];
  bin_to_hex(hex, output, res);
  hex[res * 2] = '\0';
  mmOutput->Lines->Text = (char*)hex;

  delete[] input;
  delete[] output;
  delete[] hex;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ComboBoxClick(TObject *Sender)
{
  setControl();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EditChange(TObject *Sender)
{
  setControl();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::MemoChange(TObject *Sender)
{
  setControl();  
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::sbtAboutClick(TObject *Sender)
{
  if (AboutForm == NULL) AboutForm = new TAboutForm(Application);
  AboutForm->ShowModal();
}
//---------------------------------------------------------------------------

