#include "aria.hpp"

extern "C" {
#include "aria.h"
}

// ----------------------------------------------------------------------------
// Aria
// ----------------------------------------------------------------------------
Aria::Aria()
{
  roundKey = 0/*NULL*/;
  numberOfRounds = 0;
}

Aria::~Aria()
{
  finalize();
}

bool Aria::initialize(unsigned char* masterKey, int keyBits, bool encrypt)
{
  finalize();
  int keySize = getMasterKeySize(keyBits);
  if (keySize == 0) return false;
  numberOfRounds = getNumberOfRounds(keyBits);
  if (numberOfRounds == 0) return false;
  roundKey = new unsigned char[ARIA_NB * (numberOfRounds + 1)];
  if (encrypt)
    numberOfRounds = EncKeySetup(masterKey, roundKey, keyBits);
  else // decrypt
    numberOfRounds = DecKeySetup(masterKey, roundKey, keyBits);
  return true;
}

bool Aria::finalize()
{
  if (roundKey != 0/*NULL*/)
  {
    delete[] roundKey;
    roundKey = 0/*NULL*/;
  }
  numberOfRounds = 0;
  return true;
}

int  Aria::crypt(unsigned char*input, unsigned char* output, int len)
{
  int res = 0;

  if (len % ARIA_NB != 0) return -1;
  while (true)
  {
    if (len <= 0) break;
    if (len >= ARIA_NB)
    {
      Crypt(input, numberOfRounds, roundKey, output);
      res    += ARIA_NB;
      input  += ARIA_NB;
      output += ARIA_NB;
      len    -= ARIA_NB;
    }
  }
  return res;
}

int Aria::getMasterKeySize(int keyBits)
{
  switch (keyBits)
  {
    case 128:
    case 192:
    case 256:
      return keyBits / 8;
  }
  return 0;
}

int Aria::getNumberOfRounds(int keyBits)
{
  switch (keyBits)
  {
    case 128: return 12;
    case 192: return 14;
    case 256: return 16;
  }
  return 0;
}
