// ----------------------------------------------------------------------------
//
// ARIA crypto library module for C++ version 1.0
//
// http://www.gilgil.net
//
// Last Build : 2009.01.30
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

#ifndef __ARIA_HPP__
#define __ARIA_HPP__

// ----------------------------------------------------------------------------
// ARIA constants
// ----------------------------------------------------------------------------
#define ARIA_NB 16

// ----------------------------------------------------------------------------
// Aria
// ----------------------------------------------------------------------------
class Aria
{
protected:
  unsigned char* roundKey;
  int            numberOfRounds;
public:
  Aria();
  virtual ~Aria();
public:
  //
  // Initialize Aria cypty module. Returns true on success.
  //   masterKey : pointer of key(length == keyBits / 8).
  //   keyBits   : 128, 192 or 256.
  //   encrypt   : true-enctypt / false-decrypt
  bool initialize(unsigned char* masterKey, int keyBits, bool encrypt);

  //
  // Finalize Aria crtyp module. Returns true on success.
  //
  bool finalize();

  //
  // Encrypt or decrypt(input > output).
  //   len       : buffer length of input and output(multiple of 16).
  //
  int  crypt(unsigned char*input, unsigned char* output, int len);
public:

  //
  // Returns key size by given key bit count.
  //
  static int getMasterKeySize(int keyBits);

  //
  // Returns Nr(number of round) by given key bit count.
  //
  static int getNumberOfRounds(int keyBits);
};

#endif  __ARIA_HPP__
